@@ -1,48 +1,45 @@
-"""
-Copyright (c) 2014 Rebecca R. Murphy
-All rights reserved.
+# Copyright (c) 2014 Rebecca R. Murphy
+# All rights reserved.
 
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
+# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
+# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 
-3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
+# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-"""
-
-import array
 import os
 import sys
 import struct
-import math as ma
 import csv
 
 try:
-    from scipy.optimize import curve_fit
-    import matplotlib
     import numpy as np
     from matplotlib import pyplot as plt
     from mpl_toolkits.mplot3d import Axes3D
     from matplotlib import cm
-
-
-    # matplotlib customizations
-    matplotlib.rc('xtick', labelsize=18) 
-    matplotlib.rc('ytick', labelsize=18) 
-    font = {'family' : 'sans',
-            'weight' : 'normal',
-            'size'   : 18}
-
-    matplotlib.rc('font', **font)
+    import matplotlib
+    from scipy.optimize import curve_fit
+    import math as ma
+    print "Imports all good"
 except:
     print "Numerical libraries not present in your systems"
-    pass
 
 
+def _runningMean(x, T):
+    """
+    Calculate running sum over array x with summation window T.
+    Called by APBS and DCBS to calculate number of photons arriving within time window T
+
+    Arguments:
+    * x: a numpy array of photon counts
+    * T: window length (number of array entries) over which to sum photons
+    """
+    return np.convolve(x, np.ones((T,)), mode='same')
+
 class FRET_data:
 
     """
@@ -66,8 +63,8 @@ class FRET_data:
 
         """
         self.donor = np.array(donor).astype(float)
-        self.acceptor = np.array(acceptor).astype(float)
-
+        self.acceptor = np.array(acceptor).astype(float)    
+    
     def subtract_bckd(self, bckd_d, bckd_a):
         """
         Subtract background noise from donor and acceptor channel data.
@@ -165,6 +162,152 @@ class FRET_data:
         E = self.acceptor / (self.acceptor + (gamma * self.donor))
         return E
 
+    # burst search
+    def APBS(self, T, M, L):
+        """
+        All-photon bust search algorithm as implemented in Nir et al. J Phys Chem B. 2006 110(44):22103-24.
+        Returns a FRET_bursts object.
+
+        Arguments:
+        * T: time-window (in bins) over which to sum photons 
+        * M: number of photons in window of length T required to identify a potential burst. 
+        * L: total number of photons required for an identified burst to be accepted.
+
+        From Nir et al.:
+        The start (respectively, the end) of a potential burst is detected when the number of photons in the averaging window of duration T is larger (respectively, smaller) than the minimum number of photons M.
+        A potential burst is retained if the number of photons it contains is larger than a minimum number L.
+        """
+
+        data = self.donor + self.acceptor
+        means = _runningMean(data, T)
+        donor_bursts, acceptor_bursts, b_start, b_end = self._APBS_bursts(data, means, M, L)
+        FRET_bursts_obj = FRET_bursts(donor_bursts, acceptor_bursts, b_start, b_end)
+        return FRET_bursts_obj
+
+
+    def _APBS_bursts(self, data, means, M, L):
+        assert len(data) == len(means)
+        donor_bursts = []
+        acceptor_bursts = []
+        burst_starts = []
+        burst_ends = []
+        positions = []
+        bursts = []
+        burst = 0
+        b_donor = 0
+        b_acceptor = 0
+
+        burst_len = 0
+        collecting = False
+        for pos in range(len(data)):
+            if means[pos] >= M:
+                collecting = True
+                burst_len += 1
+                burst += data[pos]
+                b_donor += self.donor[pos]
+                b_acceptor += self.acceptor[pos]
+            else:
+                if collecting:
+                    bursts.append(burst)
+                    donor_bursts.append(b_donor)
+                    acceptor_bursts.append(b_acceptor)
+                    burst_pos = pos - (float(burst_len) / 2)
+                    burst_starts.append(pos - burst_len)
+                    burst_ends.append(pos)
+                    positions.append(burst_pos)
+                burst = 0
+                b_donor = 0
+                b_acceptor = 0
+                burst_len = 0
+                collecting = False
+
+        bursts = np.array(bursts)
+        donor_bursts = np.array(donor_bursts)
+        acceptor_bursts = np.array(acceptor_bursts)
+        burst_starts = np.array(burst_starts)
+        burst_ends = np.array(burst_ends)
+        positions = np.array(positions)
+
+        select = bursts >= L
+        bursts = bursts[select]
+        positions = positions[select]
+        donor_bursts = donor_bursts[select]
+        acceptor_bursts = acceptor_bursts[select]
+        burst_starts = burst_starts[select]
+        burst_ends = burst_ends[select]
+
+        return donor_bursts, acceptor_bursts, burst_starts, burst_ends
+
+    def DCBS(self, T, M, L):
+        """
+        Dual-channel bust search algorithm as implemented in Nir et al. J Phys Chem B. 2006 110(44):22103-24.
+        Returns a FRET_bursts object.
+
+        Arguments:
+        * T: time-window (in bins) over which to sum photons 
+        * M: number of photons in window of length T required to identify a potential burst. 
+        * L: total number of photons required for an identified burst to be accepted.
+
+        From Nir et al.:
+        The start (respectively, the end) of a potential burst is detected when the number of photons in the averaging window of duration T is larger (respectively, smaller) than the minimum number of photons M.
+        A potential burst is retained if the number of photons it contains is larger than a minimum number L.
+        """
+
+        donor_means = _runningMean(self.donor, T)
+        acceptor_means = _runningMean(self.acceptor, T)
+        donor_bursts, acceptor_bursts, b_start, b_end = self._DCBS_bursts(donor_means, acceptor_means, M, L)
+        FRET_bursts_obj = FRET_bursts(donor_bursts, acceptor_bursts, b_start, b_end)
+        return FRET_bursts_obj
+
+    def _DCBS_bursts(self, donor_means, acceptor_means, M, L):
+        assert len(self.donor) == len(donor_means)
+        donor_bursts = []
+        acceptor_bursts = []
+        burst_starts = []
+        burst_ends = []
+        positions = []
+        b_donor = 0
+        b_acceptor = 0
+
+        burst_len = 0
+        collecting = False
+        for pos in range(len(self.donor)):
+            # condition depends on both channels
+            if (donor_means[pos] >= M) & (acceptor_means[pos] >= M):
+                collecting = True
+                burst_len += 1
+                b_donor += self.donor[pos]
+                b_acceptor += self.acceptor[pos]
+            else:
+                if collecting:
+                    donor_bursts.append(b_donor)
+                    acceptor_bursts.append(b_acceptor)
+                    burst_pos = pos - (float(burst_len) / 2)
+                    burst_starts.append(pos - burst_len)
+                    burst_ends.append(pos)
+                    positions.append(burst_pos)
+                b_donor = 0
+                b_acceptor = 0
+                burst_len = 0
+                collecting = False
+
+        donor_bursts = np.array(donor_bursts)
+        acceptor_bursts = np.array(acceptor_bursts)
+        burst_starts = np.array(burst_starts)
+        burst_ends = np.array(burst_ends)
+        positions = np.array(positions)
+
+        select = (donor_bursts >= L) & (acceptor_bursts >= L) 
+        positions = positions[select]
+        donor_bursts = donor_bursts[select]
+        acceptor_bursts = acceptor_bursts[select]
+        burst_starts = burst_starts[select]
+        burst_ends = burst_ends[select]
+
+        return donor_bursts, acceptor_bursts, burst_starts, burst_ends
+
+
+
     def make_3d_plot(self, filepath, imgname, imgtype="pdf", labels = ["Donor", "Acceptor", "Frequency"]):
         """
         Make a 3D histogram of donor and acceptor photon counts.
@@ -236,7 +379,7 @@ class FRET_data:
         plt.savefig(figname)
         plt.close()
         return self
-
+    
     def build_histogram(self, filepath, csvname, gamma=1.0, bin_min=0.0, bin_max=1.0, bin_width=0.02, image = False, imgname = None, imgtype=None, gauss = True, gaussname = None):
         """
         Build a proximity ratio histogram and save the frequencies and bin centres as a csv file. 
@@ -317,6 +460,59 @@ class FRET_data:
         plt.close()
         return self
 
+class FRET_bursts(FRET_data):
+    """
+    This class holds single molecule burst data. Photon bursts are stored in numpy arrays. There is a separate array for each of the two
+    photon streams, for the start and end of each burst and for the burst duration.
+
+    The two attributes corresponding to bursts from the four photon streams from a FRET experiment are numpy arrays:
+
+    * donor: The donor channel
+    * acceptor: The acceptor channel
+
+    The three further attributes, corresponding to burst duration, burst start time and burst end time are also numpy arrays:
+
+    * burst_len: Length (in bins) of each identified burst
+    * burst_starts: Start time (bin number) of each identified burst
+    * burst_ends: End time (bn number) of each identified burst
+    
+    The class can be initialized directly from four lists or arrays: two of burst photon counts; the burst start times and the burst end times: 
+    bursts = FRET_bursts(donor_bursts, acceptor_bursts, burst_starts, burst_ends).
+    
+    However, it is more typically achieved by running the APBS or DCBS algorithm that forms part of the FRET_data class.
+
+    """
+
+
+    def __init__(self, donor, acceptor, burst_starts, burst_ends):
+        FRET_data.__init__(self, donor, acceptor)
+        self.total = self.donor + self.acceptor
+        self.burst_starts = burst_starts
+        self.burst_ends = burst_ends
+        self.burst_len = self.burst_ends - self.burst_starts
+
+    def scatter_intensity(self, filepath, imgname, imgtype="pdf", labels = ["Burst Duration", "Burst Intensity"]):
+        """
+        Plot a scatter plot of burst brightness vs burst duration
+
+        Arguments:
+        * filepath: file path to the directory in which the image will be saved
+        * imgname: name under which the image will be saved
+
+        Keyword arguments:
+        * imgtype: filetype of histogram image. Accepted values: jpg, tiff, rgba, png, ps, svg, eps, pdf
+        * labels: labels for x and y axes, as a 2-element list of strings: ["x-title", "y-title"]. Default value: ["Burst Duration", "Burst Intensity"]
+        """
+
+        fullname = ".".join([imgname, imgtype])
+        figname = os.path.join(filepath, fullname)
+        plt.scatter(self.burst_len, self.total)
+        plt.xlabel(labels[0])
+        plt.ylabel(labels[1])
+        plt.savefig(figname)
+        plt.close()
+        return self
+
 
 def parse_csv(filepath, filelist, delimiter=","):
     """
@@ -411,6 +607,5 @@ def _fit_gauss(bin_centres, frequencies, p0=[1.0, 0.0, 1.0]):
     hist_fit = gauss(bin_centres, *coeff)
     return coeff, var_matrix, hist_fit
 
-
 if __name__ == "__main__":
     pass
\ No newline at end of file
